﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Threading;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("LinearLeaf", "A leaf type that uses linear models as leaf models. This is the standard for M5' regression")]
  public class LinearLeaf : ParameterizedNamedItem, ILeafType<IConfidenceRegressionModel> {
    #region Constructors & Cloning
    [StorableConstructor]
    private LinearLeaf(bool deserializing) : base(deserializing) { }
    private LinearLeaf(LinearLeaf original, Cloner cloner) : base(original, cloner) { }
    public LinearLeaf() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LinearLeaf(this, cloner);
    }
    #endregion

    #region IModelType
    public IConfidenceRegressionModel BuildModel(IRegressionProblemData pd, IRandom random, CancellationToken cancellation, out int noParameters) {
      if (pd.Dataset.Rows < MinLeafSize(pd)) throw new ArgumentException("The number of training instances is too small to create a linear model");
      double rmse, cvRmse;
      noParameters = pd.AllowedInputVariables.Count() + 1;
      return PreconstructedLinearModel.CreateConfidenceLinearModel(pd, out rmse, out cvRmse);
    }

    public int MinLeafSize(IRegressionProblemData pd) {
      return pd.AllowedInputVariables.Count() + 2;
    }
    #endregion
  }
}