#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Threading;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;

namespace HeuristicLab.Algorithms.DataAnalysis {
  internal static class M5StaticUtilities {
    public static ResultCollection RunSubAlgorithm(IAlgorithm alg, int random, CancellationToken cancellation) {
      if (alg.Parameters.ContainsKey("SetSeedRandomly") && alg.Parameters.ContainsKey("Seed")) {
        var seed = alg.Parameters["Seed"].ActualValue as IntValue;
        var setSeed = alg.Parameters["SetSeedRandomly"].ActualValue as BoolValue;
        if (seed == null || setSeed == null)
          throw new ArgumentException("The parameters SetSeedRandomly and Seed do not have the expected type");
        setSeed.Value = false;
        seed.Value = random;
      }
      if (alg.ExecutionState != ExecutionState.Paused) alg.Prepare();
      alg.Start(cancellation);
      return alg.Results;
    }
  }
}